clear
clc
clear global

load OB_Reduced_Wage_Order_4.mat

%% Draw Shock Paths
epsilon=[eps_L;eps_L;zeros(T-2,1)];
theta=zeros(T,1);

for t=1:T
    if t==1
        theta(t)=1^rho*exp(epsilon(t));
    else
        theta(t)=theta(t-1)^rho*exp(epsilon(t));
    end
end

thetaLprime=theta.^rho.*exp(eps_L);

%% Solve once to recover stochastic steady state
for tt=1:T   

    if tt==T/2
        disp('50 percent complete')
    end

    if tt==1
        state=[k(tt) n(tt) theta(tt) ss];
    else
        state=[k(tt) n(tt) theta(tt) s(tt-1)];
    end
        
    data_temp=Hermite(state,order);
    data_1=[1 data_temp];

    gam_1=gam_g(1:length(gam_g)/4);
    gam_2=gam_g(1+length(gam_g)/4:2*(length(gam_g)/4));
    gam_3=gam_g(1+2*(length(gam_g)/4):3*(length(gam_g)/4));
    gam_4=gam_g(1+3*(length(gam_g)/4):length(gam_g));
        
    g1(tt)=data_1*gam_1;
    g2(tt)=data_1*gam_2;
    g3(tt)=data_1*gam_3;
    g4(tt)=data_1*gam_4;

    if tt==1
        n_lag(tt)=ns;
    else
        n_lag(tt)=n(tt-1);
    end
        
    y(tt)=theta(tt)*k(tt)^alpha*n_lag(tt)^(1-alpha);
    Y(tt)=ph*y(tt);
        
    c(tt)=1/g1(tt);
      
    v(tt)=(c(tt)*g3(tt))/G;
    v(tt)=max(v(tt),0.00001);
        
    w(tt)=c(tt)*(g4(tt)+eta);
      
    match(tt)=((1-n(tt))*v(tt))/(((1-n(tt))^gamma+v(tt)^gamma)^...
        (1/gamma));
    n(tt+1)=(1-xs)*n(tt)+match(tt);
    Phi(tt)=v(tt)/(1-n(tt));
    f(tt)=match(tt)/(1-n(tt));
    q(tt)=match(tt)/v(tt);        

    k(tt+1)=Y(tt)-c(tt)+(1-delta)*k(tt)-v(tt)*G;

    p(tt)=beta*c(tt)*ph*k(tt+1)^alpha*n(tt)^(1-alpha)*g2(tt);
       
    s(tt)=(alpha*p(tt)+w(tt)*n(tt)+v(tt)*G)/((1+alpha)*p(tt));
    r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
    lambda(tt)=1;
    Mu(tt)=0;
            
    if dp*(1-s(tt))*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)<A*s(tt)
        s(tt)=(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha))/...
            (A+dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha));
        r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
        lambda(tt)=1+((r(tt)*k(tt+1)-alpha*(1-s(tt))*p(tt))/(alpha*...
            (1-s(tt))*p(tt)*s(tt)-r(tt)*k(tt+1)));
        Mu(tt)=((p(tt)*s(tt))/(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)))*...
            (lambda(tt)-1);
    end
          
    Jn(tt)=(1-alpha)*(1-s(tt))*p(tt)*(1/n(tt))+lambda(tt)*((1-xs)*G*(1/q(tt))-w(tt))+...
        Mu(tt)*(1-alpha)*dp*(1-s(tt))*thetaLprime(tt)*(k(tt+1)/n(tt))^alpha;
    Vn(tt)=(b*Jn(tt))/((1-b)*c(tt));
        
end

ss_stoch=s(end);
ns_stoch=n(end);
ks_stoch=k(end);

k=[ks_stoch;zeros(T,1)];
n=[ns_stoch;zeros(T,1)];
%% Solve again using stochastic steady state
for tt=1:T   

    if tt==T/2
        disp('50 percent complete')
    end

    if tt==1
        state=[k(tt) n(tt) theta(tt) ss_stoch];
    else
        state=[k(tt) n(tt) theta(tt) s(tt-1)];
    end
        
    data_temp=Hermite(state,order);
    data_1=[1 data_temp];

    gam_1=gam_g(1:length(gam_g)/4);
    gam_2=gam_g(1+length(gam_g)/4:2*(length(gam_g)/4));
    gam_3=gam_g(1+2*(length(gam_g)/4):3*(length(gam_g)/4));
    gam_4=gam_g(1+3*(length(gam_g)/4):length(gam_g));
        
    g1(tt)=data_1*gam_1;
    g2(tt)=data_1*gam_2;
    g3(tt)=data_1*gam_3;
    g4(tt)=data_1*gam_4;

    if tt==1
        n_lag(tt)=ns_stoch;
    else
        n_lag(tt)=n(tt-1);
    end
        
    y(tt)=theta(tt)*k(tt)^alpha*n_lag(tt)^(1-alpha);
    Y(tt)=ph*y(tt);
        
    c(tt)=1/g1(tt);
      
    v(tt)=(c(tt)*g3(tt))/G;
    v(tt)=max(v(tt),0.00001);
        
    w(tt)=c(tt)*(g4(tt)+eta);
      
    match(tt)=((1-n(tt))*v(tt))/(((1-n(tt))^gamma+v(tt)^gamma)^...
        (1/gamma));
    n(tt+1)=(1-xs)*n(tt)+match(tt);
    Phi(tt)=v(tt)/(1-n(tt));
    f(tt)=match(tt)/(1-n(tt));
    q(tt)=match(tt)/v(tt);        

    k(tt+1)=Y(tt)-c(tt)+(1-delta)*k(tt)-v(tt)*G;

    p(tt)=beta*c(tt)*ph*k(tt+1)^alpha*n(tt)^(1-alpha)*g2(tt);
       
    s(tt)=(alpha*p(tt)+w(tt)*n(tt)+v(tt)*G)/((1+alpha)*p(tt));
    r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
    lambda(tt)=1;
    Mu(tt)=0;
            
    if dp*(1-s(tt))*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)<A*s(tt)
        s(tt)=(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha))/...
            (A+dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha));
        r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
        lambda(tt)=1+((r(tt)*k(tt+1)-alpha*(1-s(tt))*p(tt))/(alpha*...
            (1-s(tt))*p(tt)*s(tt)-r(tt)*k(tt+1)));
        Mu(tt)=((p(tt)*s(tt))/(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)))*...
            (lambda(tt)-1);
    end
          
    Jn(tt)=(1-alpha)*(1-s(tt))*p(tt)*(1/n(tt))+lambda(tt)*((1-xs)*G*(1/q(tt))-w(tt))+...
        Mu(tt)*(1-alpha)*dp*(1-s(tt))*thetaLprime(tt)*(k(tt+1)/n(tt))^alpha;
    Vn(tt)=(b*Jn(tt))/((1-b)*c(tt));
        
end

save OB_ReducedWageEqu_IRF.mat