clear
clc
clear global

load OB_start_Order_4.mat
gam_g=[gam1;gam2;gam3;gam4];

imag_tol=1e-10;

oldopts=optimset('fsolve');
newopts=optimset('Display','iter');
options=optimset(oldopts,newopts);

RS=0;

T=10000;

scale=0.9615;

% Agency Problem Parameters
ph=0.9;
pl=0.4;
dp=ph-pl;

% Productivity Shock Parameters
rho=0.9325;                             % Target Auto(ALP)
sigma=0.0101;                           % Target Vol(ALP)
ths=1;
eps_L=-2.5*sigma;
eps_H=2.5*sigma;

% Generate nodes and weights for quadrature integration
[n_nodes,nodes,weights]=GH_Quadrature(9,1,sigma^2);

% Remaining Parameters
alpha=1/3;
beta=0.99;
delta=0.02;
b=0.014;                           % Target E_{w,ALP}=0.449

us=0.055;
fs=0.6225;
Phis=0.75;

%% Solve for Steady state
rs=1-beta*(1-delta);

ns=1-us;
xs=(us*fs)/(1-us);
matchs=xs*ns;
vs=Phis*us;
qs=fs/Phis;

gam_guess=6;
gam_pars=[us,vs,matchs];
gamma=fsolve(@(gam_guess) Mfunct(gam_guess,gam_pars),gam_guess,options);

RepRatio=0.992;

guess=[10;1];
pars=[ph,alpha,beta,delta,us,fs,gamma,b,Phis,RepRatio];
NewVals=fsolve(@(guess) SS(guess,pars),guess,options);

ks=NewVals(1);
Gs=NewVals(2);

Ys=ph*ks^alpha*ns^(1-alpha);
ps=beta*Ys;
ss=1-((rs*ks)/(alpha*ps));
ws=(ps*ss-rs*ks-vs*Gs)/ns;
Jns=Gs/(qs*beta);
cs=Ys-delta*ks-vs*Gs;
eta=RepRatio*(ws/cs);
Vns=(b*Jns)/((1-b)*cs);

A=scale*dp*((1-ss)/ss)*(1^rho*exp(eps_L))*ks^alpha*ns^(1-alpha);

G=Gs;

display([RepRatio,eta,G,(vs*G)/(ws*ns)]);

%% Preallocate Variables
k=[ks;zeros(T,1)];
n=[ns;zeros(T,1)];

v=zeros(T,1);
f=zeros(T,1);
q=zeros(T,1);
Phi=zeros(T,1);
y=zeros(T,1);
Y=zeros(T,1);
r=zeros(T,1);
c=zeros(T,1);
w=zeros(T,1);
match=zeros(T,1);
Jn=zeros(T,1);
Vn=zeros(T,1);
s=zeros(T,1);
p=zeros(T,1);

g1=zeros(T,1);
g2=zeros(T,1);
g3=zeros(T,1);
g4=zeros(T,1);

lambda=zeros(T,1);
Mu=ones(T,1);

n_lag=zeros(T,1);
s_NB=zeros(T,1);

exflag=zeros(T,1);

exp1=zeros(T,1);
exp2=zeros(T,1);
exp3=zeros(T,1);
exp4=zeros(T,1);

rhs1=zeros(T,1);
rhs2=zeros(T,1);
rhs3=zeros(T,1);
rhs4=zeros(T,1);

%% Draw Shock Paths
randn('state',5)
epsilon=sigma*randn(T,1);

pos_L=find(epsilon<eps_L);
epsilon(pos_L)=eps_L;

pos_H=find(epsilon>eps_H);
epsilon(pos_H)=eps_H;

theta=zeros(T,1);

for t=1:T
    if t==1
        theta(t)=1^rho*exp(epsilon(t));
    else
        theta(t)=theta(t-1)^rho*exp(epsilon(t));
    end
end

thetaLprime=theta.^rho.*exp(eps_L);

oldopts=optimset('fsolve');
newopts=optimset('Display','off');
options=optimset(oldopts,newopts);

% PEA Parameters
maxiters=1000;
lamb=0.75;
diff=100;
tol=1e-008;  
iters=0;
largest_max2=5;

%% Preallocate things created in while loop 
par_change_pos=zeros(maxiters,1);
par_change_base_val=zeros(maxiters,1);

max_k_diff=ones(maxiters,1);
mean_k_diff=ones(maxiters,1);

max_n_diff=ones(maxiters,1);
mean_n_diff=ones(maxiters,1);

max_s_diff=ones(maxiters,1);
mean_s_diff=ones(maxiters,1);

max_g1_diff=ones(maxiters,1);
mean_g1_diff=ones(maxiters,1);

max_g2_diff=ones(maxiters,1);
mean_g2_diff=ones(maxiters,1);

max_g3_diff=ones(maxiters,1);
mean_g3_diff=ones(maxiters,1);

max_g4_diff=ones(maxiters,1);
mean_g4_diff=ones(maxiters,1);

largest_max=ones(maxiters,1);
largest_mean=ones(maxiters,1);

check_running=ones(maxiters,1);

MSE1=ones(maxiters,1);
MSE2=ones(maxiters,1);
MSE3=ones(maxiters,1);
MSE4=ones(maxiters,1);

r2_1=ones(maxiters,1);
r2_2=ones(maxiters,1);
r2_3=ones(maxiters,1);
r2_4=ones(maxiters,1);

ALPvol=ones(maxiters,1);
ALPcorr=ones(maxiters,1);
Elast=ones(maxiters,1);
Avg_Tight=ones(maxiters,1);
Avg_Find=ones(maxiters,1);

if RS==1
    loadit=['load OB_' int2str(T/1000) 'k.mat'];
    eval(loadit);
end

while (diff>tol)&&(iters<maxiters)
    
    iters=iters+1;
    disp(['Iteration ' int2str(iters) ':']);
    
    if iters>1
        gam_g=lamb*gam_g+(1-lamb)*gam_n;
    end

    if max(abs(imag(gam_g)))<imag_tol
        gam_g=real(gam_g);
    else
        lkjdkjf
    end

    for tt=1:T   

        if tt==T/2
            disp('50 percent complete')
        end

        if tt==1
            state=[k(tt) n(tt) theta(tt) ss];
        else
            state=[k(tt) n(tt) theta(tt) s(tt-1)];
        end
        
        data_temp=Hermite(state,order);
        data_1=[1 data_temp];

        gam_1=gam_g(1:length(gam_g)/4);
        gam_2=gam_g(1+length(gam_g)/4:2*(length(gam_g)/4));
        gam_3=gam_g(1+2*(length(gam_g)/4):3*(length(gam_g)/4));
        gam_4=gam_g(1+3*(length(gam_g)/4):length(gam_g));
        
        g1(tt)=data_1*gam_1;
        g2(tt)=data_1*gam_2;
        g3(tt)=data_1*gam_3;
        g4(tt)=data_1*gam_4;

        if tt==1
            n_lag(tt)=ns;
        else
            n_lag(tt)=n(tt-1);
        end
        
        y(tt)=theta(tt)*k(tt)^alpha*n_lag(tt)^(1-alpha);
        Y(tt)=ph*y(tt);
        
        c(tt)=1/g1(tt);
        
        v(tt)=(c(tt)*g3(tt))/G;
        v(tt)=max(v(tt),0.00001);
        
        w(tt)=c(tt)*(g4(tt)+eta);
        
        match(tt)=((1-n(tt))*v(tt))/(((1-n(tt))^gamma+v(tt)^gamma)^...
            (1/gamma));
        n(tt+1)=(1-xs)*n(tt)+match(tt);
        Phi(tt)=v(tt)/(1-n(tt));
        f(tt)=match(tt)/(1-n(tt));
        q(tt)=match(tt)/v(tt);        

        k(tt+1)=Y(tt)-c(tt)+(1-delta)*k(tt)-v(tt)*G;

        p(tt)=beta*c(tt)*ph*k(tt+1)^alpha*n(tt)^(1-alpha)*g2(tt);
        
        s(tt)=(alpha*p(tt)+w(tt)*n(tt)+v(tt)*G)/((1+alpha)*p(tt));
        r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
        lambda(tt)=1;
        Mu(tt)=0;
        
        % To make sure the Binding s satisfies the models logic
        s_NB(tt)=s(tt);
        
        if dp*(1-s(tt))*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)<A*s(tt)
            s(tt)=(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha))/...
                (A+dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha));
            r(tt)=(p(tt)*s(tt)-w(tt)*n(tt)-v(tt)*G)/k(tt+1);
            lambda(tt)=1+((r(tt)*k(tt+1)-alpha*(1-s(tt))*p(tt))/(alpha*...
                (1-s(tt))*p(tt)*s(tt)-r(tt)*k(tt+1)));
            Mu(tt)=((p(tt)*s(tt))/(dp*thetaLprime(tt)*k(tt+1)^alpha*n(tt)^(1-alpha)))*...
                (lambda(tt)-1);
        end
        
        if (s_NB(tt)-s(tt)<0)||(lambda(tt)<1)||(Mu(tt)<0)
            disp('Toggle Error!')
            dfadfke
        end
       
        Jn(tt)=(1-alpha)*(1-s(tt))*p(tt)*(1/n(tt))+lambda(tt)*((1-xs)*G*(1/q(tt))-w(tt))+...
            Mu(tt)*(1-alpha)*dp*(1-s(tt))*thetaLprime(tt)*(k(tt+1)/n(tt))^alpha;
        Vn(tt)=(b*Jn(tt))/((1-b)*c(tt));
        
        comp_check=imag([g1(tt) g2(tt) g3(tt) g4(tt) c(tt) v(tt) w(tt)...
            match(tt) n(tt) Phi(tt) f(tt) q(tt) k(tt) p(tt) s(tt) r(tt)...
            lambda(tt) Mu(tt) Jn(tt) Vn(tt) y(tt) Y(tt)]);

        if max(abs(comp_check))<imag_tol
            g1(tt)=real(g1(tt));
            g2(tt)=real(g2(tt));
            g3(tt)=real(g3(tt));
            g4(tt)=real(g4(tt));
            c(tt)=real(c(tt));
            v(tt)=real(v(tt));
            w(tt)=real(w(tt));
            match(tt)=real(match(tt));
            n(tt)=real(n(tt));
            Phi(tt)=real(Phi(tt));
            f(tt)=real(f(tt));
            q(tt)=real(q(tt));
            k(tt)=real(k(tt));
            p(tt)=real(p(tt));
            s(tt)=real(s(tt));
            r(tt)=real(r(tt));
            lambda(tt)=real(lambda(tt));
            Mu(tt)=real(Mu(tt));
            Jn(tt)=real(Jn(tt));
            Vn(tt)=real(Vn(tt));
            y(tt)=real(y(tt));
            Y(tt)=real(Y(tt));
        else
            disp('Imaginary Problem')
            kdfljd
        end
       
        next_theta=theta(tt).^rho.*exp(nodes);
        
        thetaL2p=next_theta.^rho*exp(eps_L);
        
        data_temp=[ones(n_nodes,1)*k(tt+1) ones(n_nodes,1)*n(tt+1)...
            next_theta ones(n_nodes,1)*s(tt)];
        data_temp=Hermite(data_temp,order);
        data=[ones(n_nodes,1) data_temp];
        
        g1_prime=data*gam_1;
        g2_prime=data*gam_2;
        g3_prime=data*gam_3;
        g4_prime=data*gam_4;
        
        cp=1./g1_prime;
        vp=(cp.*g3_prime)./G;
        
        neg_v_pos=find(vp<=0);
        vp(neg_v_pos)=0.00001;
        
        wp=cp.*(g4_prime+eta);
        
        matchp=((1-n(tt+1)).*vp)./(((1-n(tt+1)).^gamma+vp.^gamma).^(1./gamma));
        n2p=(1-xs).*n(tt+1)+matchp;
        Phip=vp./(1-n(tt+1));
        fp=matchp./(1-n(tt+1));
        qp=matchp./vp;
        
        k2p=ph.*next_theta.*k(tt+1).^alpha.*n(tt).^(1-alpha)-cp+(1-delta).*k(tt+1)-vp.*G;
        pp=beta.*cp.*ph.*k2p.^alpha.*n(tt+1).^(1-alpha).*g2_prime;
        sp=(alpha.*pp+wp.*n(tt+1)+vp.*G)./((1+alpha).*pp);
             
        for gg=1:n_nodes
            if dp*(1-sp(gg))*thetaL2p(gg)*k2p(gg)^alpha*n(tt+1)^(1-alpha)<A*sp(gg)
                sp(gg)=(dp*thetaL2p(gg)*k2p(gg)^alpha*n(tt+1)^(1-alpha))/...
                    (A+dp*thetaL2p(gg)*k2p(gg)^alpha*n(tt+1)^(1-alpha));
            end
        end
        
        rp=(pp.*sp-wp.*n(tt+1)-vp.*G)./k2p;
        lambdap=1+((rp.*k2p-alpha.*(1-sp).*pp)./(alpha.*(1-sp).*pp.*sp-rp.*k2p));
        Mup=((pp.*sp)./(dp.*thetaL2p.*k2p.^alpha.*n(tt+1).^(1-alpha))).*(lambdap-1);
        
        Jnp=(1-alpha).*(1-sp).*pp.*(1./n(tt+1))+lambdap.*((1-xs).*G.*(1./qp)-wp)+...
            Mup.*(1-alpha).*dp.*(1-sp).*thetaL2p.*(k2p./n(tt+1)).^alpha;
        Vnp=(b.*Jnp)./((1-b).*cp);
        
        exp1(tt)=((r(tt)/c(tt))+(beta*(1-delta))./cp)'*weights;
        rhs1(tt)=1/c(tt);
        
        if abs(imag(exp1(tt)))<imag_tol
            exp1(tt)=real(exp1(tt));
        else
            kdlkjf
        end
        
        exp2(tt)=(next_theta./cp)'*weights;
        rhs2(tt)=p(tt)/(beta*c(tt)*ph*k(tt+1)^alpha*n(tt)^(1-alpha));
        
        if abs(imag(exp2(tt)))<imag_tol
            exp2(tt)=real(exp2(tt));
        else
            kdlkjf
        end
        
        exp3(tt)=((beta./cp).*((match(tt).*Jnp)./lambda(tt)))'*weights;
        rhs3(tt)=(v(tt)*G)/c(tt);
        
        if abs(imag(exp3(tt)))<imag_tol
            exp3(tt)=real(exp3(tt));
        else
            kdlkjf
        end
        
        exp4(tt)=(Vn(tt)-beta.*(1-xs-f(tt)).*Vnp)'*weights;
        rhs4(tt)=(w(tt)/c(tt))-eta;
        
        if abs(imag(exp4(tt)))<imag_tol
            exp4(tt)=real(exp4(tt));
        else
            kdlkjf
        end
        
    end 
        
    exp1_Adj=(exp1-mean(exp1))/std(exp1);
    exp2_Adj=(exp2-mean(exp2))/std(exp2);
    exp3_Adj=(exp3-mean(exp3))/std(exp3);
    exp4_Adj=(exp4-mean(exp4))/std(exp4);
    
    data_T=[k(1:T) n(1:T) theta [ss;s(1:T-1)]];
    Herm_Poly=Hermite(data_T,order);
    Herm_Poly_Adj=(Herm_Poly-ones(T,1)*mean(Herm_Poly))./(ones(T,1)*std(Herm_Poly));

    [U,S,V]=svd(Herm_Poly_Adj,0);

    Inv_S=diag(1./diag(S));
    UT=U';

    b1_Adj=V*Inv_S*UT*exp1_Adj;
    b2_Adj=V*Inv_S*UT*exp2_Adj;
    b3_Adj=V*Inv_S*UT*exp3_Adj;
    b4_Adj=V*Inv_S*UT*exp4_Adj;

    gam_n1=zeros(size(Herm_Poly,2)+1,1);
    gam_n2=zeros(size(Herm_Poly,2)+1,1);
    gam_n3=zeros(size(Herm_Poly,2)+1,1);
    gam_n4=zeros(size(Herm_Poly,2)+1,1);
    
    for t=1:size(Herm_Poly,2)
    
        gam_n1(t+1)=(std(exp1)/std(Herm_Poly(:,t)))*b1_Adj(t);
        gam_n2(t+1)=(std(exp2)/std(Herm_Poly(:,t)))*b2_Adj(t);
        gam_n3(t+1)=(std(exp3)/std(Herm_Poly(:,t)))*b3_Adj(t);
        gam_n4(t+1)=(std(exp4)/std(Herm_Poly(:,t)))*b4_Adj(t);
        
    end

    gam_n1(1)=mean(exp1)-mean(Herm_Poly)*gam_n1(2:end);
    gam_n2(1)=mean(exp2)-mean(Herm_Poly)*gam_n2(2:end);
    gam_n3(1)=mean(exp3)-mean(Herm_Poly)*gam_n3(2:end);   
    gam_n4(1)=mean(exp4)-mean(Herm_Poly)*gam_n4(2:end);
    
    gam_n=[gam_n1;gam_n2;gam_n3;gam_n4];
    
    if max(abs(imag(gam_n)))<imag_tol
        gam_n=real(gam_n);
    else
        dkdlkjd
    end
    
    check=norm(gam_g-gam_n);
    check_running(iters)=check;
    
    if iters==1
        k_old=k;
        n_old=n;
        s_old=s;
    else
             
        kdiff=mean(abs(1-k./k_old));
        ndiff=mean(abs(1-n./n_old));
        s_lag_diff=mean(abs(1-s./s_old));
        
        diff=max([kdiff,ndiff,s_lag_diff]);

        k_old=k;
        n_old=n;
        s_old=s;
        
    end
    
    ALP=([1;theta(1:T-1)]./theta).*(Y./[ns;n(1:T-1)]);
    ALPc=log(ALP)-hpfilter(log(ALP),1600);
   
    wc=log(w)-hpfilter(log(w),1600);
    
    Elast=regress(wc,ALPc);
    
    ALPc2=log(ALP)-hpfilter(log(ALP),10^5);
    ALPvol(iters)=std(ALPc2)*100;
    ALPcorr(iters)=regress(ALPc2(1:end-1),ALPc2(2:end));    
    
    bind_rate=length(find(lambda>2*mean(lambda)-min(lambda)))/T;
    
    Avg_Tight(iters)=mean(Phi);
    Avg_Find(iters)=mean(f);
    
    disp(['*Check ' num2str(check,5)...
        '    diff ' num2str(diff,5)...
        '    Elast ' num2str(Elast)...
        '    vol(Y)' num2str(ALPvol(iters))...
        '    Corr(ALP)' num2str(ALPcorr(iters))...
        '    Binding ' num2str(bind_rate)...
        '    Mean(Phi)' num2str(Avg_Tight(iters))...
        '    Mean(f)' num2str(Avg_Find(iters))])
    
    savefile=['OB_Reduced_Wage_Order_' int2str(order) '_Restart.mat'];
    save(savefile,'gam_g','gam_n','largest_max2','k_old','n_old','s_old',...
        'iters','check_running','order');
    
end 

saveit=['save OB_Reduced_Wage_Order_' int2str(order) '.mat'];
eval(saveit);
    














